SystemOrganization addCategory: #'GT-Debugger'!
SystemOrganization addCategory: 'GT-Debugger-Actions'!
SystemOrganization addCategory: 'GT-Debugger-Model'!
SystemOrganization addCategory: 'GT-Debugger-UI'!

Trait named: #GTBrowsingActions
	uses: {}
	category: 'GT-Debugger-Actions'!

!GTBrowsingActions commentStamp: 'AndreiChis 12/30/2015 11:21' prior: 0!
I provide various actions for browsing a Context object. These actions are loaded by the debugger in the context menu of a stack.!

!GTBrowsingActions methodsFor: 'actions' stamp: 'AndreiChis 4/16/2014 18:01'!
browseClassRefs

	self currentContext ifNotNil: [ :aContext || class|
		class := aContext methodClass theNonMetaClass.
		class isTrait 
			ifTrue: [ self systemNavigation browseAllUsersOfTrait: class ]
			ifFalse: [ self systemNavigation browseAllCallsOnClass: class ] ]
! !

!GTBrowsingActions methodsFor: 'actions' stamp: 'AndreiChis 4/16/2014 18:01'!
browseClassVarRefs
	"1/17/96 sw: devolve responsibility to the class, so that the code that does the real work can be shared"

	| cls |
	cls := self currentClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation  browseClassVarRefs: cls]! !

!GTBrowsingActions methodsFor: 'actions' stamp: 'AndreiChis 4/16/2014 18:01'!
browseClassVariables
	"Browse the class variables of the selected class."
	| cls |
	cls := self currentClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation  browseClassVariables: cls]
! !

!GTBrowsingActions methodsFor: 'actions' stamp: 'AndreiChis 4/16/2014 18:00'!
browseInstVarRefs
	"1/26/96 sw: real work moved to class, so it can be shared"
	| cls |
	cls := self currentClassOrMetaClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation browseInstVarRefs: cls]! !

!GTBrowsingActions methodsFor: 'actions' stamp: 'AndreiChis 4/16/2014 17:59'!
browseMessages
	"Present a menu of all messages sent by the current message. 
	Open a message set browser of all implementors of the message chosen."

	self currentMessageName ifNotNil: [   
		self systemNavigation browseAllImplementorsOf: self currentMessageName ]! !

!GTBrowsingActions methodsFor: 'actions' stamp: 'AndreiChis 8/17/2015 17:21'!
browseMethodFull
	"Create and schedule a full Browser and then select the current class and message."

	self currentClassOrMetaClass ifNotNil: [ 
		Smalltalk tools browser 
			openOnClass: self currentClassOrMetaClass
			selector: self currentMessageName ]! !

!GTBrowsingActions methodsFor: 'actions' stamp: 'AndreiChis 4/16/2014 17:58'!
browseSendersOfMessages
	"Present a menu of the current message, as well as all messages sent by it.  Open a message set browser of all senders of the selector chosen."

	self currentMessageName ifNotNil: [   
		self systemNavigation browseAllSendersOf: self currentMessageName ]! !

!GTBrowsingActions methodsFor: 'actions' stamp: 'VincentBlondeau 1/28/2015 16:12'!
browseVersions
	"Create and schedule a message set browser on all versions of the 
	currently selected message selector."

	Smalltalk tools versionBrowser browseVersionsForClass: self currentClassOrMetaClass selector: self currentMessageName! !

!GTBrowsingActions methodsFor: 'accessing' stamp: 'AndreiChis 4/16/2014 18:03'!
currentClass

	^ self currentContext methodClass! !

!GTBrowsingActions methodsFor: 'accessing' stamp: 'AndreiChis 1/15/2015 16:25'!
currentClassOrMetaClass

	^ self currentContext methodClass! !

!GTBrowsingActions methodsFor: 'accessing' stamp: 'AndreiChis 4/16/2014 18:03'!
currentContext
	
	^ self debugger currentContext! !

!GTBrowsingActions methodsFor: 'accessing' stamp: 'AndreiChis 4/16/2014 18:03'!
currentMessageCategoryName
	"Answer the name of the message category of the message of the currently selected context."

	^ self currentClass organization categoryOfElement: self currentMessageName! !

!GTBrowsingActions methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2015 19:16'!
currentMessageName

	^ self currentContext methodSelector! !

!GTBrowsingActions methodsFor: 'actions' stamp: 'AndreiChis 4/16/2014 18:02'!
fileOutMessage
	"Put a description of the selected message on a file"

	self currentMessageName ifNotNil: [
		Cursor write showWhile:
			[ self currentClassOrMetaClass fileOutMethod: self currentMessageName ] ]! !

!GTBrowsingActions methodsFor: 'actions' stamp: 'AndreiChis 4/16/2014 18:02'!
inspectInstances
	"Inspect all instances of the selected class."

	self currentClassOrMetaClass ifNotNil: [ :aClass |
		aClass theNonMetaClass inspectAllInstances ]! !

!GTBrowsingActions methodsFor: 'actions' stamp: 'AndreiChis 4/16/2014 18:02'!
inspectSubInstances
	"Inspect all instances of the selected class and all its subclasses"

	self currentClassOrMetaClass ifNotNil: [ :aClass |
		aClass theNonMetaClass inspectSubInstances ]! !

!GTBrowsingActions methodsFor: 'actions' stamp: 'AndreiChis 4/16/2014 17:59'!
methodHierarchy
	"Create and schedule a method browser on the hierarchy of implementors."

	self systemNavigation 
			methodHierarchyBrowserForClass: self currentClassOrMetaClass 
			selector: self currentMessageName
! !

GLMCompositePresentation subclass: #GTDebuggerVariablesBrowser
	instanceVariableNames: 'debugger'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-UI'!

!GTDebuggerVariablesBrowser commentStamp: 'AndreiChis 12/30/2015 11:06' prior: 0!
I am a special browser that is used in the first pane of the inspector that is embedded in the debugger. Its main intention is to provide a unified list of variables that come from:
- the instance variables of the receiver
- the local variables from the context!

!GTDebuggerVariablesBrowser methodsFor: 'testing' stamp: 'AndreiChis 6/13/2014 14:30'!
compareVariableTypeOf: x withVariable: y in: aContext

	^ (self variableTypeOf: x in: aContext) < (self variableTypeOf: y in: aContext)! !

!GTDebuggerVariablesBrowser methodsFor: 'building' stamp: 'AndreiChis 2/9/2016 17:39'!
compose  
	"self title: [:each | GTInspector new printObjectAsAnItem: each receiver ]."
	"self morphicAct: [:anotherComposite | anotherComposite entity receiver browse]
				icon: GLMUIThemeExtraIcons glamorousBrowse 
				entitled: 'Browse'."
	self tabulator with: [:browser | 
		browser row: #variables.
		browser transmit to: #variables; andShow: [:a | 
			self variablesIn: a ].
		browser transmit 
			from: #variables port: #entity; 
			to: #variables port: #selection; 
			when: [ :aContext | self debugger selectedVariableName notNil and: [aContext notNil] ];
			transformed: [ :aContext | self selectedVariableForContext: aContext ].
		browser transmit from: #variables; toOutsidePort: #selection; transformed: [ :pair | pair value ] ]! !

!GTDebuggerVariablesBrowser methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 18:12'!
debugger
	^ debugger! !

!GTDebuggerVariablesBrowser methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 18:12'!
debugger: anObject
	debugger := anObject! !

!GTDebuggerVariablesBrowser methodsFor: 'building' stamp: 'AndreiChis 1/10/2016 18:41'!
selectedVariableForContext: aContext 
	|variablesMappings|
	
	variablesMappings := self variablesFromContext: aContext.
	^ variablesMappings 
		detect: [ :each | each key = self debugger selectedVariableName ] 
		ifNone: [ variablesMappings detect: [ :each | each key = 'self' ] ifNone: [ nil ] ]! !

!GTDebuggerVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 6/13/2014 14:09'!
tagsForVariable: aString from: aContext
	^ (aContext reservedNames includes: aString) 
		ifTrue: [ #() ]
		ifFalse: [ self variableTypeOf: aString in: aContext ]
			
	! !

!GTDebuggerVariablesBrowser methodsFor: 'building' stamp: 'AndreiChis 1/14/2015 19:47'!
variableTypeOf: aString in: aContext
	^ (aContext receiver class classThatDefinesInstanceVariable: aString) notNil
			ifTrue: [ 'attribute' ]
			ifFalse: [ (aContext method argumentNames includes: aString)
				ifTrue: [ 'parameter' ]
				ifFalse: [ 'temp' ] ]
			
	! !

!GTDebuggerVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 1/14/2015 19:06'!
variablesFromContext: aContext 

	^ aContext gtDebuggerRetrieveVariableValuePairs! !

!GTDebuggerVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 2/9/2016 17:39'!
variablesIn: composite
	^ composite table
		showOnly: 50;
		shouldValidate: false;
		allowsRecursiveSearch: false;
		display: [ :context | self variablesFromContext: context ];
		column: 'Type'
			evaluated: [ '' ]
			width: 75
			tags: [ :assoc :context |
				self tagsForVariable: assoc key from: context ]
			sortedBy: [ :x :y | 
				self 
					compareVariableTypeOf: x key
					withVariable: y key
					in: ((self presentations first paneNamed: #variables) port: #entity) value];
		column: 'Variable' 
			evaluated: [:assoc | 
			[ Text fromString: assoc key asString ]
					on: Error
					do: [ Text string: 'error printing' attribute: TextColor red ] ]
			width: 175;
		column: 'Value' evaluated: [:assoc | 
			[ Text fromString: (assoc value printString contractTo: 50) ]
					on: Error
					do: [ Text string: 'error printing' attribute: TextColor red ] ];
		"send: #value;"
		morphicSelectionAct: [:list | list selection value browse ] 
			icon: GLMUIThemeExtraIcons glamorousBrowse 
			on: $b 
			entitled: 'Browse';
		morphicSelectionAct: [:list | list selection value inspect ] 
			icon: GLMUIThemeExtraIcons glamorousInspect 
			on: $i 
			entitled: 'Inspect';
		onChangeOfPort: #rawSelection 
			act: [ :aPresentation | 
				self debugger selectedVariableName: aPresentation rawSelection key ]! !

GLMCompositePresentation subclass: #GTMoldableDebugger
	uses: GTBrowsingActions
	instanceVariableNames: ''
	classVariableNames: 'EnableDebuggerWindowDistinctColor EnableStackColoring GTDebuggerEnabledStatus'
	poolDictionaries: ''
	category: 'GT-Debugger-UI'!

!GTMoldableDebugger commentStamp: 'AndreiChis 12/30/2015 11:06' prior: 0!
I provide the basic functionality needed to create new debuggers. 
Subclasses should override the following methods:
	debuggerStructureIn: -> provides the structure of the debugger
	debuggerTransmissionsIn: -> creates transmissions between the widgets of a debuggers
	selectedContext !

GTMoldableDebugger class
	uses: GTBrowsingActions classTrait
	instanceVariableNames: ''!

GTMoldableDebugger subclass: #GTGenericStackDebugger
	instanceVariableNames: 'selectedVariableName'
	classVariableNames: 'AlwaysOpenFullDebugger ErrorRecursion FilterCommonMessageSends LogDebuggerStackToFile LogFileName'
	poolDictionaries: ''
	category: 'GT-Debugger-UI'!

!GTGenericStackDebugger commentStamp: 'AndreiChis 12/30/2015 11:06' prior: 0!
I am a browser that provides Smalltalk debugging functionality.
 !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 12/16/2013 16:37'!
alwaysOpenFullDebugger
	^ AlwaysOpenFullDebugger ifNil: [AlwaysOpenFullDebugger := false]! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 12/16/2013 16:37'!
alwaysOpenFullDebugger: aBoolean
	AlwaysOpenFullDebugger := aBoolean.! !

!GTGenericStackDebugger class methodsFor: 'utilities api' stamp: 'AndreiChis 12/17/2013 11:23'!
closeAllDebuggers
	"GTGenericStackDebugger closeAllDebuggers"
	(SystemWindow allSubInstances select: [:w | 
		 w model isKindOf: GTGenericStackDebugger])
			do: [:w | w delete ]! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 12/16/2013 16:37'!
defaultLogFileName
	^ 'PharoDebug.log'! !

!GTGenericStackDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 17:42'!
defaultTitle
	^ 'GT'! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 12/16/2013 16:37'!
filterCommonMessageSends
	self flag: 'horrible hack not to break the setting browser'.
	^ false! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 12/16/2013 16:37'!
filterCommonMessageSends: aBoolean
	FilterCommonMessageSends := aBoolean! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 1/3/2014 18:19'!
filterDoItSelectors
	^ false! !

!GTGenericStackDebugger class methodsFor: 'accessing - ancient preference selectors' stamp: 'TudorGirba 3/16/2015 15:05'!
filterDoItSelectors: aBoolean
	self flag: 'just for backward compatibility'! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 1/3/2014 18:19'!
filterKernelClasses
	^ false! !

!GTGenericStackDebugger class methodsFor: 'accessing - ancient preference selectors' stamp: 'TudorGirba 3/16/2015 15:05'!
filterKernelClasses: aBoolean
	self flag: 'just for backward compatibility'! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'JurajKubelka 6/22/2015 15:47'!
filterLinkSelectors
	^ false! !

!GTGenericStackDebugger class methodsFor: 'accessing - ancient preference selectors' stamp: 'JurajKubelka 6/22/2015 15:47'!
filterLinkSelectors: aBoolean 
	self flag: 'just for backward compatibility'! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 1/3/2014 18:20'!
filterNilSelectors
	^ false! !

!GTGenericStackDebugger class methodsFor: 'accessing - ancient preference selectors' stamp: 'TudorGirba 3/16/2015 15:05'!
filterNilSelectors: aBoolean 
	self flag: 'just for backward compatibility'! !

!GTGenericStackDebugger class methodsFor: 'testing' stamp: 'AndreiChis 12/16/2013 16:37'!
handlesContext: aContext

	^ true! !

!GTGenericStackDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 1/24/2016 23:56'!
helpClass

	^ GTDebuggerHelp! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 12/16/2013 16:37'!
logDebuggerStackToFile
	^ LogDebuggerStackToFile ifNil: [LogDebuggerStackToFile := true]! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 12/16/2013 16:37'!
logDebuggerStackToFile: aBoolean
	 LogDebuggerStackToFile := aBoolean! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 12/16/2013 16:37'!
logFileName
	^ LogFileName ifNil: [ self defaultLogFileName ] ! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 12/16/2013 16:37'!
logFileName: newName
	LogFileName := newName! !

!GTGenericStackDebugger class methodsFor: 'tools registry' stamp: 'AndreiChis 12/31/2013 21:01'!
register

	self registerToolsOn: Smalltalk tools.
	self registerExtraToolsOn: Smalltalk tools.
! !

!GTGenericStackDebugger class methodsFor: 'tools registry' stamp: 'AndreiChis 12/29/2015 17:40'!
registerExtraToolsOn: registry
	"This adds self in the list of tools that appears in the small debugger"
	registry registerDebugger: self withRank: 3.! !

!GTGenericStackDebugger class methodsFor: 'tools registry' stamp: 'AndreiChis 12/16/2013 16:37'!
registerToolsOn: registry
	registry register: self as: #debugger.
! !

!GTGenericStackDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 16:37'!
sessionClass

	^ DebugSession ! !

!GTGenericStackDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 12/16/2013 16:37'!
stackWidgetClass
	self flag: 'horrible hack not to break the setting browser'.
	^ self! !

!GTGenericStackDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 1/14/2015 18:55'!
variablesBrowserClass

	^ GTDebuggerVariablesBrowser! !

!GTGenericStackDebugger methodsFor: 'callbacks' stamp: 'AndreiChis 1/10/2016 19:00'!
actOnInspectorPaneRemoved: ann 

	ann browser panes size == 1 ifTrue: [ 
		self selectedVariableName: nil ]! !

!GTGenericStackDebugger methodsFor: 'building actions' stamp: 'AndreiChis 12/16/2013 16:37'!
codeActionsPragmas

	^ #( codeDebuggingAction )! !

!GTGenericStackDebugger methodsFor: 'accessing' stamp: 'AndreiChis 4/17/2014 00:36'!
codePane

	^ self browser paneNamed: #code! !

!GTGenericStackDebugger methodsFor: 'accessing' stamp: 'AndreiChis 4/17/2014 10:26'!
codePresentation
	
	^ self codePane presentations first! !

!GTGenericStackDebugger methodsFor: 'building' stamp: 'TudorGirba 1/9/2016 22:13'!
debuggerStructureIn: browser
	
	browser
		row: #stack;
		row: #code span: 2;
		row: #inspector.
		! !

!GTGenericStackDebugger methodsFor: 'building' stamp: 'AndreiChis 11/30/2015 23:30'!
debuggerTransmissionsIn: browser

	browser transmit
		fromOutsideEntityPort;
		to: #stack;
		andShow: [ :composite | self stackIn: composite ].
	browser transmit
		from: #stack port: #entity;
		to: #stack port: #selection;
		when: [ :session | session notNil ];
		transformed: [:session | self contextToSelectFrom: session ].
	browser transmit "need a method call when the selection is changed"
		from: #stack port: #selection;
		to: #stack port: #action;
		transformed: [ :context | 
			self selectionChanged: context ].
	browser transmit
		from: #stack port: #selection;
		to: #code;
		andShow: [ :composite :aContext | self methodCodeIn: composite forContext: aContext ].
	browser transmit
		from: #stack port: #selection;
		to: #inspector;
		andShow: [ :composite | self inspectorIn: composite ] 
! !

!GTGenericStackDebugger methodsFor: 'building actions' stamp: 'AndreiChis 12/16/2013 16:37'!
debuggingActionsPragmas

	^ #( debuggingAction )! !

!GTGenericStackDebugger methodsFor: 'printing/formatting' stamp: 'AndreiChis 1/12/2016 13:57'!
formatStackClassColumnForContext: aContext

	^ Text 
		string: (self printStackClassColumnForContext: aContext )
		attributes: (self textAttributesForStackEntry: aContext) 
	 ! !

!GTGenericStackDebugger methodsFor: 'printing/formatting' stamp: 'AndreiChis 1/12/2016 13:58'!
formatStackExtraColumnForContext: aContext 

	^ Text 
		string: (self printStackExtraColumnForContext: aContext )
		attributes: (self textAttributesForStackEntry: aContext) ! !

!GTGenericStackDebugger methodsFor: 'printing/formatting' stamp: 'AndreiChis 1/12/2016 13:59'!
formatStackMethodColumnForContext: aContext 

	^ Text 
		string: (self printStackMethodColumnForContext: aContext )
		attributes: (self textAttributesForStackEntry: aContext) ! !

!GTGenericStackDebugger methodsFor: 'actions' stamp: 'AndreiChis 11/5/2014 18:25'!
inspect: anObject

	"Use silentValue: instead of value: as just using value: causes the current inspector to refresh the content of it current presentations without rebuiling them (the dynamic presentation contained by the inspector does not rebuild its content)"
	((self browser paneNamed: #inspector) port: #entity) silentValue: anObject.
	
	(self browser paneNamed: #inspector) update.
	! !

!GTGenericStackDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 11/5/2014 18:22'!
inspectorIn: aComposite

	^ aComposite dynamic 
		display: [:anObject |
			self inspectorIn: GLMCompositePresentation new on: anObject ]! !

!GTGenericStackDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 1/10/2016 18:35'!
inspectorIn: aComposite on: anObject
				
	^ anObject gtConstructDebuggerInspectorIn: aComposite for: self
	
	! !

!GTGenericStackDebugger methodsFor: 'building actions' stamp: 'AndreiChis 12/16/2013 16:37'!
installCodeActionsFor: aPresentation
	
	self installActionsForRootPragmas: self codeActionsPragmas in: aPresentation! !

!GTGenericStackDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/10/2016 15:20'!
installDebuggingActionsFor: aPresentation

	self installActionsForRootPragmas: self debuggingActionsPragmas in: aPresentation! !

!GTGenericStackDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/20/2015 22:09'!
installStackDebuggingActionsFor: aPresentation

	self 
		installActionsForRootPragmas: self stackDebuggingActionsPragmas 
		in: aPresentation! !

!GTGenericStackDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 10/8/2015 20:43'!
methodCodeIn: composite forContext: aContext

	(self methodCodeWidgetIn: composite forContext: aContext)
		initialize: [ :code | 
			code selectionInterval: (self session selectedCodeRangeForContext: self currentContext) ];
		with: [ :code | self installCodeActionsFor: code ]! !

!GTGenericStackDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 10/8/2015 19:50'!
methodCodeWidgetIn: composite forContext: aContext

	^ composite pharoMethod
		title: 'Source';
		format: [ aContext sourceCode ];
		smalltalkClass: [  aContext methodClass ];
		doItReceiver: [ aContext receiver ];
		doItContext: [  aContext ]! !

!GTGenericStackDebugger methodsFor: 'opening' stamp: 'AndreiChis 11/25/2015 11:57'!
openWithFullView
	"Create and schedule a full debugger with the given label. Do not
	terminate the current active process."
	
	self open! !

!GTGenericStackDebugger methodsFor: 'opening' stamp: 'AndreiChis 11/25/2015 17:00'!
openWithNotification: aString

	self openWithFullView! !

!GTGenericStackDebugger methodsFor: 'printing/formatting' stamp: 'AndreiChis 1/12/2016 13:57'!
printStackClassColumnForContext: aContext
	| outer class methodClass |
	
	outer := aContext outerMostContext.
	^ String streamContents: [ :aStream |
		class := outer receiver class.
		aStream nextPutAll: class name.
		methodClass := outer methodClass.
		methodClass == class
			ifFalse:
				[ aStream 
						nextPut: $(;
						nextPutAll: methodClass name;
						nextPut: $) ] ]  ! !

!GTGenericStackDebugger methodsFor: 'printing/formatting' stamp: 'AndreiChis 1/12/2016 13:57'!
printStackExtraColumnForContext: aContext 

	| selector |
	^ aContext outerContext 
		ifNil: [ 
			String streamContents: [ :aStream |
				selector := aContext selector ifNil: [ aContext method defaultSelector ].
				selector = #doesNotUnderstand:
					ifTrue:[ 
						aStream space.
						(aContext tempAt: 1) selector printOn: aStream ]
					ifFalse: [ '' ] ] ]
		ifNotNil: [ aContext closure printString  ]! !

!GTGenericStackDebugger methodsFor: 'printing/formatting' stamp: 'AndreiChis 1/12/2016 13:59'!
printStackMethodColumnForContext: aContext 
	| outer selector |
	
	outer := aContext outerMostContext.
	^ String streamContents: [ :aStream |
		selector := outer selector ifNil: [ outer method defaultSelector ].
		aStream nextPutAll: selector ]! !

!GTGenericStackDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 8/21/2014 12:27'!
retrieveStackFrom: aSession

	^ aSession stack
		! !

!GTGenericStackDebugger methodsFor: 'updating' stamp: 'AndreiChis 4/18/2014 16:08'!
selectTopContext
	((self browser paneNamed: #stack) port: #selection) value: self interruptedContext! !

!GTGenericStackDebugger methodsFor: 'accessing context' stamp: 'AndreiChis 2/15/2016 20:25'!
selectedContext

	^ self browser ifNotNil: [
		((self browser paneNamed: #stack) port: #selection) value ]
	! !

!GTGenericStackDebugger methodsFor: 'accessing' stamp: 'AndreiChis 4/17/2014 00:42'!
selectedText

	^ (self codePane port: #selectedText) value! !

!GTGenericStackDebugger methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 18:07'!
selectedVariableName
	^ selectedVariableName! !

!GTGenericStackDebugger methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 18:07'!
selectedVariableName: anObject
	selectedVariableName := anObject! !

!GTGenericStackDebugger methodsFor: 'updating' stamp: 'AndreiChis 2/16/2016 15:09'!
selectionChanged: aContext
	
	self browser ifNil: [ ^ self ].
	UIManager default defer: [ 
		(self browser paneNamed: #stack) presentations updateToolbar ].
	
	self class enableStackColoring ifFalse: [ ^ self ].  

	UIManager default defer: [
		(self browser paneNamed: #stack) presentations presentations ifNotEmpty: [ :p | 
			self session interruptedProcess ifNotNil: [ p first update ] ] ]! !

!GTGenericStackDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 16:37'!
selectionInterval

	^ self selectionIntervalForPane: #code! !

!GTGenericStackDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 16:37'!
selectionInterval: anInterval

	^ ((self browser paneNamed: #code) port: #selectionInterval) value: anInterval! !

!GTGenericStackDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 16:37'!
selectionIntervalForPane: aSymbol

	^ ((self browser paneNamed: aSymbol) port: #selectionInterval) value ifNil: [ ^ 1 to: 0 ]! !

!GTGenericStackDebugger methodsFor: 'old api' stamp: 'AndreiChis 12/16/2013 16:37'!
send
	
	self session stepInto: self currentContext.
	self updateBrowser.
! !

!GTGenericStackDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 16:37'!
sourceCode: aString

	^ ((self browser paneNamed: #code) port: #text) value: aString! !

!GTGenericStackDebugger methodsFor: 'building actions' stamp: 'AndreiChis 1/10/2016 15:20'!
stackDebuggingActionsPragmas

	^ #( stackDebuggingAction )
! !

!GTGenericStackDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 1/20/2015 22:34'!
stackIn: composite

	^ (self stackWidgetIn: composite)
		with: [ :stackWidget | self installStackDebuggingActionsFor: stackWidget ]! !

!GTGenericStackDebugger methodsFor: 'accessing' stamp: 'AndreiChis 1/8/2016 15:58'!
stackPresentation
	
	^ (self browser paneNamed: #stack) presentations first! !

!GTGenericStackDebugger methodsFor: 'building presentations' stamp: 'TudorGirba 1/21/2016 23:07'!
stackWidgetIn: composite

	^ composite fastTable	
		title: 'Stack';
		display: [:aSession | self retrieveStackFrom: aSession];
		column: 'Class' 
			evaluated: [ :aContext | self formatStackClassColumnForContext: aContext]
			width: 300;
		column: 'Method' 
			evaluated: [ :aContext | self formatStackMethodColumnForContext: aContext ];
		column: 'Other' 
			evaluated: [ :aContext | self formatStackExtraColumnForContext: aContext ];
		hideHeader! !

!GTGenericStackDebugger methodsFor: 'old api' stamp: 'AndreiChis 1/12/2016 16:19'!
sugsContext
	^SugsDebuggerContext model: self! !

!GTGenericStackDebugger methodsFor: 'printing/formatting' stamp: 'MarianoMartinezPeck 12/29/2015 18:21'!
textAttributesForStackEntry: aContext

	(self selectedContext isNil or: self class enableStackColoring not) 
		ifTrue: [ ^ {TextColor color: Smalltalk ui theme textColor} ].

	^ Smalltalk ui theme styleContext: aContext from: self.! !

!GTGenericStackDebugger methodsFor: 'updating' stamp: 'AndreiChis 10/8/2015 20:55'!
updateBrowser
	super updateBrowser.
	self updateSelectionInterval.! !

!GTGenericStackDebugger methodsFor: 'updating' stamp: 'AndreiChis 1/11/2016 13:12'!
updateCodeEditor
	self codePane update.
	self updateSelectionInterval.! !

!GTGenericStackDebugger methodsFor: 'updating' stamp: 'AndreiChis 11/30/2015 23:32'!
updateSelectionInterval

	self selectedContext ifNil: [ ^ self ].
	self selectionInterval: (self session pcRangeForContext:  self selectedContext )! !

!GTGenericStackDebugger methodsFor: 'accessing' stamp: 'AndreiChis 8/16/2014 12:25'!
window
	"The default debugger from Pharo defines this method.
	Some tools use it and break. Try to return the window
	if possible. Kind of a hack."
	^ self dependents ifNotNil: [ :dependentsArray |
		dependentsArray ifNotEmpty: [ 
			(dependentsArray at: 1) isSystemWindow
				ifTrue: [ dependentsArray at: 1 ]
				ifFalse: [ nil ] ] ]! !

!GTMoldableDebugger class methodsFor: 'testing' stamp: 'AndreiChis 11/27/2015 18:49'!
availableAutomatically
	"Should this debugger be taken into account when looking for a debugger to handle an exception."
	
	^ false! !

!GTMoldableDebugger class methodsFor: 'instance creation' stamp: 'AndreiChis 12/16/2013 16:39'!
debugSession: aSession

	<primitive: 19> "simulation guard"

	"WorldState addDeferredUIMessage: ["
		^ self openOn: aSession 
	"]"! !

!GTMoldableDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 16:38'!
defaultTitle
	self subclassResponsability! !

!GTMoldableDebugger class methodsFor: 'settings api' stamp: 'TudorGirba 2/11/2014 06:35'!
enableDebuggerWindowDistinctColor
	^ EnableDebuggerWindowDistinctColor ifNil: [EnableDebuggerWindowDistinctColor := false]! !

!GTMoldableDebugger class methodsFor: 'settings api' stamp: 'TudorGirba 2/11/2014 06:35'!
enableDebuggerWindowDistinctColor: aBoolean
	EnableDebuggerWindowDistinctColor := aBoolean.! !

!GTMoldableDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 1/7/2016 10:11'!
enableStackColoring
	^ EnableStackColoring ifNil: [EnableStackColoring := false]! !

!GTMoldableDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 1/7/2016 10:11'!
enableStackColoring: aBoolean
	EnableStackColoring := aBoolean.! !

!GTMoldableDebugger class methodsFor: 'testing' stamp: 'AndreiChis 12/16/2013 16:40'!
handlesContext: aContext
	self subclassResponsibility ! !

!GTMoldableDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 1/25/2016 11:06'!
helpClass

	self subclassResponsibility! !

!GTMoldableDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 1/7/2016 10:11'!
isGTDebuggerEnabled

	^ GTDebuggerEnabledStatus ifNil: [ GTDebuggerEnabledStatus := Smalltalk tools debugger = self ]! !

!GTMoldableDebugger class methodsFor: 'instance creation api' stamp: 'AndreiChis 11/27/2015 15:41'!
on: aDebugSession

	^ self new 	
			session: aDebugSession! !

!GTMoldableDebugger class methodsFor: 'opening api' stamp: 'AndreiChis 11/27/2015 15:41'!
openOn: aDebugSession withFullView: aBool
	
	self openOn: aDebugSession withFullView: aBool andNotification: nil! !

!GTMoldableDebugger class methodsFor: 'opening api' stamp: 'AndreiChis 11/27/2015 19:00'!
openOn: aDebugSession withFullView: aBool andNotification: aString
	
	| debugger |
	debugger := (Smalltalk tools 
		debuggerDispatcherFor: aDebugSession interruptedContext
		matching: [:aDebugger | aDebugger availableAutomatically ]) 
			ifNil: [ 
				"Use the current debugger"
				self on: aDebugSession ]
			ifNotNil: [ :aDebugger | 
				"Use a custom extension"
				aDebugger on: (aDebugger spanNewSessionFrom: aDebugSession) ].
	
	aBool | self alwaysOpenFullDebugger
		ifTrue: [ debugger openWithFullView ]
		ifFalse: [ debugger openWithNotification: aString]! !

!GTMoldableDebugger class methodsFor: 'window color api' stamp: 'AndreiChis 12/16/2013 16:41'!
patchworkUIThemeColor
	"Answer a default color for UI themes that make use of different colors for Browser, MessageList etc..."

	^ Color lightRed
! !

!GTMoldableDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 16:39'!
sessionClass
	self subclassResponsability! !

!GTMoldableDebugger class methodsFor: 'settings api' stamp: 'AndreiChis 1/7/2016 10:11'!
setGTDebuggerEnabledStatus: aBoolean
	| oldStatus |
	
	oldStatus := self isGTDebuggerEnabled.
	GTDebuggerEnabledStatus := aBoolean.
	oldStatus ~= GTDebuggerEnabledStatus ifTrue: [ 
		GTDebuggerEnabledStatus 
			ifTrue: [ self register ]
			ifFalse: [ SpecDebugger registerToolsOn: Smalltalk tools ] ]
	
	! !

!GTMoldableDebugger class methodsFor: 'private' stamp: 'AndreiChis 11/25/2015 16:05'!
spanNewSessionForContext: aContext fromProcess: aProcess

	^ self sessionClass 
		named: 'temporary name' on: aProcess startedAt: aContext! !

!GTMoldableDebugger class methodsFor: 'accessing' stamp: 'DenisKudryashov 2/17/2016 10:56'!
spanNewSessionFrom: anotherSession

	^ (self sessionClass 
		named: anotherSession name 
		on: anotherSession interruptedProcess 
		startedAt: anotherSession interruptedContext)
			errorWasInUIProcess: anotherSession errorWasInUIProcess;
			yourself! !

!GTMoldableDebugger class methodsFor: 'private' stamp: 'AndreiChis 11/25/2015 16:54'!
spanNewSessionLabeled: aString forContext: aContext fromProcess: aProcess

	^ self sessionClass 
		named: aString on: aProcess startedAt: aContext! !

!GTMoldableDebugger class methodsFor: 'icons' stamp: 'AndreiChis 4/16/2014 18:28'!
taskbarIcon

	^ Smalltalk ui icons smallDebugIcon! !

!GTMoldableDebugger methodsFor: 'callbacks' stamp: 'AndreiChis 12/16/2013 16:47'!
actOnBrowserClosing: ann
	self session ifNotNil: [ :aSession | aSession terminate ]! !

!GTMoldableDebugger methodsFor: 'callbacks' stamp: 'AndreiChis 1/10/2016 18:57'!
actOnInspectorPaneRemoved: ann! !

!GTMoldableDebugger methodsFor: 'building actions' stamp: 'AndreiChis 12/16/2013 16:46'!
actionsForPragmas: aSymbolsCollection

	^ (self allActionsForPragmas: aSymbolsCollection)
				collect: [ :aDebugAction | aDebugAction asGlamourAction ]
! !

!GTMoldableDebugger methodsFor: 'building actions' stamp: 'AndreiChis 12/16/2013 17:25'!
allActionsForPragmas: aSymbolsCollection
	
	^ (self session class 
		debuggingActionsForPragmas: aSymbolsCollection 
		for: self)

	
! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
browseClassRefs

	self currentContext ifNotNil: [ :aContext || class|
		class := aContext methodClass theNonMetaClass.
		class isTrait 
			ifTrue: [ self systemNavigation browseAllUsersOfTrait: class ]
			ifFalse: [ self systemNavigation browseAllCallsOnClass: class ] ]
! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
browseClassVarRefs
	"1/17/96 sw: devolve responsibility to the class, so that the code that does the real work can be shared"

	| cls |
	cls := self currentClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation  browseClassVarRefs: cls]! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
browseClassVariables
	"Browse the class variables of the selected class."
	| cls |
	cls := self currentClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation  browseClassVariables: cls]
! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
browseInstVarRefs
	"1/26/96 sw: real work moved to class, so it can be shared"
	| cls |
	cls := self currentClassOrMetaClass.
	(cls notNil and: [cls isTrait not])
		ifTrue: [self systemNavigation browseInstVarRefs: cls]! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
browseMessages
	"Present a menu of all messages sent by the current message. 
	Open a message set browser of all implementors of the message chosen."

	self currentMessageName ifNotNil: [   
		self systemNavigation browseAllImplementorsOf: self currentMessageName ]! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
browseMethodFull
	"Create and schedule a full Browser and then select the current class and message."

	self currentClassOrMetaClass ifNotNil: [ 
		Smalltalk tools browser 
			openOnClass: self currentClassOrMetaClass
			selector: self currentMessageName ]! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
browseSendersOfMessages
	"Present a menu of the current message, as well as all messages sent by it.  Open a message set browser of all senders of the selector chosen."

	self currentMessageName ifNotNil: [   
		self systemNavigation browseAllSendersOf: self currentMessageName ]! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
browseVersions
	"Create and schedule a message set browser on all versions of the 
	currently selected message selector."

	Smalltalk tools versionBrowser browseVersionsForClass: self currentClassOrMetaClass selector: self currentMessageName! !

!GTMoldableDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 16:45'!
browser 

	^ self presentations first! !

!GTMoldableDebugger methodsFor: 'actions' stamp: 'AndreiChis 2/16/2016 13:55'!
close

	self browser ifNotNil: [ :aBrowser | aBrowser close ]! !

!GTMoldableDebugger methodsFor: 'building' stamp: 'AndreiChis 9/12/2014 14:50'!
compose
	"give the window a dedicated reddish color to ensure it gets noticed"
	self class enableDebuggerWindowDistinctColor ifTrue: [ 
		self color: (Color r: 1.0 g: 0.3 b: 0.3) ].
	self installDebuggingActionsFor: self.
	self tabulator with: [:browser | 
		self debuggerStructureIn: browser.
		self debuggerTransmissionsIn: browser ].
	self registerActions! !

!GTMoldableDebugger methodsFor: 'accessing context' stamp: 'AndreiChis 2/16/2016 13:57'!
contextToSelectFrom: aSession
	^ aSession context! !

!GTMoldableDebugger methodsFor: 'accessing' stamp: ''!
currentClass

	^ self currentContext methodClass! !

!GTMoldableDebugger methodsFor: 'accessing' stamp: ''!
currentClassOrMetaClass

	^ self currentContext methodClass! !

!GTMoldableDebugger methodsFor: 'accessing context' stamp: 'AndreiChis 2/16/2016 13:57'!
currentContext

	^self selectedContext 
		ifNil: [ self contextToSelectFrom: self session ] 
		ifNotNil:  [ self selectedContext  ]! !

!GTMoldableDebugger methodsFor: 'accessing' stamp: ''!
currentMessageCategoryName
	"Answer the name of the message category of the message of the currently selected context."

	^ self currentClass organization categoryOfElement: self currentMessageName! !

!GTMoldableDebugger methodsFor: 'accessing' stamp: ''!
currentMessageName

	^ self currentContext methodSelector! !

!GTMoldableDebugger methodsFor: 'actions' stamp: 'AndreiChis 12/29/2015 16:44'!
debug: aContext using: aDebuggerClass
	| currentSession newDebugger |
	
	currentSession := self session.
	self detachSession.
	self browser close.
	currentSession updateContextTo: aContext.
	newDebugger := aDebuggerClass on: (aDebuggerClass spanNewSessionFrom: currentSession).
	newDebugger openWithFullView.
	newDebugger window window "Hack to get this working with SpecDebugger"
		position: self window position;
		extent: self window extent
		! !

!GTMoldableDebugger methodsFor: 'building' stamp: 'AndreiChis 12/16/2013 17:03'!
debuggerStructureIn: browser
	self subclassResponsability
		! !

!GTMoldableDebugger methodsFor: 'building' stamp: 'AndreiChis 12/16/2013 17:07'!
debuggerTransmissionsIn: browser
	self subclassResponsibility ! !

!GTMoldableDebugger methodsFor: 'actions' stamp: 'AndreiChis 12/16/2013 16:46'!
detachSession
	self flag: 'why silent value?'.
	(self browser pane port: #entity) silentValue: nil! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
fileOutMessage
	"Put a description of the selected message on a file"

	self currentMessageName ifNotNil: [
		Cursor write showWhile:
			[ self currentClassOrMetaClass fileOutMethod: self currentMessageName ] ]! !

!GTMoldableDebugger methodsFor: 'protocol' stamp: 'AndreiChis 4/16/2014 23:00'!
initialExtent
"This code was taken from the standard debugger to get the same behavior. Fell free to refactor"

	RealEstateAgent standardWindowExtent y < 400 "a tiny screen" 
		ifTrue: [^ 700@500].
	
	^ [ | w h |
		w := Display width.
		h := Display height.
		(w - (w // 3)) @ (h - (h // 5)) ]
		on: Error
		do: [ 850@600 ]! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
inspectInstances
	"Inspect all instances of the selected class."

	self currentClassOrMetaClass ifNotNil: [ :aClass |
		aClass theNonMetaClass inspectAllInstances ]! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
inspectSubInstances
	"Inspect all instances of the selected class and all its subclasses"

	self currentClassOrMetaClass ifNotNil: [ :aClass |
		aClass theNonMetaClass inspectSubInstances ]! !

!GTMoldableDebugger methodsFor: 'building actions' stamp: 'AndreiChis 12/16/2013 16:46'!
installActionsForRootPragmas: aSymbolsCollection in: aPresentation

	aPresentation dynamicActions: [ :presentation |
		self actionsForPragmas: (aSymbolsCollection collect: [ :aSymbol | #gt, aSymbol capitalized ]) ] .
	
	aPresentation dynamicActionsOnSelection: [ :presentation |
		self actionsForPragmas: (aSymbolsCollection collect: [ :aSymbol | #gtContext, aSymbol capitalized ]) ] .! !

!GTMoldableDebugger methodsFor: 'building actions' stamp: 'AndreiChis 9/12/2014 14:48'!
installDebuggingActionsFor: aPresentation

	self subclassResponsibility! !

!GTMoldableDebugger methodsFor: 'accessing context' stamp: 'AndreiChis 12/16/2013 17:04'!
interruptedContext

	^ self session context! !

!GTMoldableDebugger methodsFor: 'accessing' stamp: 'AndreiChis 1/21/2014 15:28'!
interruptedProcess

	^ self session interruptedProcess! !

!GTMoldableDebugger methodsFor: 'actions' stamp: ''!
methodHierarchy
	"Create and schedule a method browser on the hierarchy of implementors."

	self systemNavigation 
			methodHierarchyBrowserForClass: self currentClassOrMetaClass 
			selector: self currentMessageName
! !

!GTMoldableDebugger methodsFor: 'scripting opening' stamp: 'AndreiChis 11/30/2015 23:19'!
openOn: anObject 
	^ (super openOn: anObject)
		extent: self initialExtent;
		title: self session name;
		yourself! !

!GTMoldableDebugger methodsFor: 'building' stamp: 'AndreiChis 1/10/2016 18:57'!
registerActions

	self when: GLMBrowserClosing send: #actOnBrowserClosing: to: self.
	self when: DebugActionExecuted send: #updateBrowserFrom: to: self.
	self when: GLMPaneRemoved send: #actOnInspectorPaneRemoved: to: self.
! !

!GTMoldableDebugger methodsFor: 'accessing context' stamp: 'AndreiChis 12/16/2013 17:05'!
selectedContext
	self subclassResponsibility ! !

!GTMoldableDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 16:45'!
session
 
	^ (self browser pane port: #entity) value! !

!GTMoldableDebugger methodsFor: 'accessing' stamp: 'AndreiChis 11/25/2015 16:51'!
session: aSession

	(self browser pane port: #entity) value: aSession.
	self title: aSession name! !

!GTMoldableDebugger methodsFor: 'accessing' stamp: 'AndreiChis 12/17/2013 10:11'!
title
	
	^ super title 
		ifNil: [ self class defaultTitle ] 
		ifNotNil: [ :aString | aString ] ! !

!GTMoldableDebugger methodsFor: 'updating' stamp: 'AndreiChis 4/1/2015 11:26'!
updateBrowser
	| browserPane session |
	
	session := self session.
	browserPane := self browser pane.
	(browserPane port: #entity) silentValue: nil.
	(browserPane port: #entity) value: session.
	browserPane update.! !

!GTMoldableDebugger methodsFor: 'updating' stamp: 'AndreiChis 11/27/2015 15:32'!
updateBrowserFrom: anAnnouncement
	
	anAnnouncement action needsUpdate ifTrue: [
		self updateBrowser ]! !

!GTMoldableDebugger methodsFor: 'callbacks' stamp: 'AndreiChis 12/16/2013 16:47'!
windowIsClosing 
	self announce: GLMBrowserClosing new! !

!RestartDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/10/2016 15:48'!
gtGlobalDebuggingActionFor: aDebugger
	<gtDebuggingAction>
	
	^ (self forDebugger: aDebugger)
		label: nil! !

!RestartDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/10/2016 15:49'!
gtStackDebuggingActionFor: aDebugger
	<gtStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousRestart! !

!UITheme methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/8/2016 19:41'!
sameClassContextStyleFor: aContext

	^ {TextColor blue}! !

!UITheme methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/8/2016 19:42'!
samePackageContextStyleFor: aContext

	^ {TextColor black}! !

!UITheme methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/8/2016 19:42'!
styleContext: aContext from: aDebugger.

	(aDebugger selectedContext == aContext) ifTrue: [ ^ {TextColor color: Smalltalk ui theme textColor}  ].

	(aContext receiver class = aDebugger selectedContext receiver class)
		ifTrue: [ ^ self sameClassContextStyleFor: aContext ].
	(aContext receiver class package = aDebugger selectedContext receiver class package)
		ifTrue: [ ^ self samePackageContextStyleFor: aContext ]. 

	^ self unrelatedContextStyleFor: aContext! !

!UITheme methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/8/2016 19:42'!
unrelatedContextStyleFor: aContext

	^ {TextColor gray}! !

!DebugAction methodsFor: '*GT-Debugger' stamp: 'TudorGirba 1/10/2016 17:15'!
asGlamourAction

	^ ((GLMGenericAction new) 
			action: [ self execute ]; 
			icon: self icon;
			title: self label;
			showTitle;
			shortcut: (self keyText ifNotNil: [:aString | aString at: 1]);
			category: self category;
			yourself)! !

DebugAction subclass: #GTBrowseMethodDebuggerAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTBrowseMethodDebuggerAction commentStamp: 'AndreiChis 12/30/2015 11:17' prior: 0!
I am a debugging action for browsing the currently selected class.!

!GTBrowseMethodDebuggerAction class methodsFor: 'registration' stamp: 'TudorGirba 11/16/2013 08:24'!
gtActionFor: aDebugger
	<gtCodeDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousBrowse! !

!GTBrowseMethodDebuggerAction methodsFor: 'accessing' stamp: 'TudorGirba 11/16/2013 08:15'!
defaultLabel

	^ 'Browse'! !

!GTBrowseMethodDebuggerAction methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2015 22:54'!
defaultOrder

	^ 220! !

!GTBrowseMethodDebuggerAction methodsFor: 'actions' stamp: 'VincentBlondeau 9/1/2015 14:19'!
executeAction
	Smalltalk tools browser 
	openOnClass: self debugger selectedContext methodClass
 	selector: self debugger selectedContext selector! !

!GTBrowseMethodDebuggerAction methodsFor: 'accessing' stamp: 'TudorGirba 11/16/2013 08:14'!
id

	^ #gtBrowseMethodDebuggerAction! !

!GTBrowseMethodDebuggerAction methodsFor: 'actions' stamp: 'AndreiChis 1/10/2016 15:39'!
initialize
	super initialize.
	self needsUpdate: false! !

DebugAction subclass: #GTChangeDebuggerAction
	instanceVariableNames: 'debuggerClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTChangeDebuggerAction commentStamp: 'AndreiChis 12/30/2015 11:24' prior: 0!
I am an action for switching to another debugger.
My class side methods register one such action for every debugger that can be used with the current execution.!

!GTChangeDebuggerAction class methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 17:50'!
changeDebuggerActionsForContext: aContext excluding: aDebugger
	
	^ (self debuggersForContext: aContext excluding: aDebugger) collect: [ :aDebuggerClass | 
		(self forDebugger: aDebugger)
			debuggerClass: aDebuggerClass;
			icon: (Form extent: 0@0) ]! !

!GTChangeDebuggerAction class methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 17:41'!
debuggersForContext: aContext excluding: aDebugger 
	"Ungly check to make sure the debugger will work until everything is in the image"
	(Smalltalk tools respondsTo: #debuggersForContext:) 
		ifTrue: [ ^ (Smalltalk tools debuggersForContext: aContext) reverse
			remove: aDebugger class ifAbsent: [ ];
			yourself]
		ifFalse: [ ^ OrderedCollection new ]! !

!GTChangeDebuggerAction class methodsFor: 'as yet unclassified' stamp: 'AndreiChis 1/8/2016 23:17'!
gtChangeDebuggerActionsFor: aDebugger
	<gtDebuggingAction>
	
	^ self 
		changeDebuggerActionsForContext: aDebugger interruptedContext 
		excluding: aDebugger! !

!GTChangeDebuggerAction methodsFor: 'accessing' stamp: 'AndreiChis 6/18/2013 17:51'!
debuggerClass: aClass

	debuggerClass := aClass.
	self label: debuggerClass defaultTitle.! !

!GTChangeDebuggerAction methodsFor: 'initialization' stamp: 'AndreiChis 8/30/2013 14:39'!
defaultCategory 
	
	^ 'Available Debuggers'! !

!GTChangeDebuggerAction methodsFor: 'initialization' stamp: 'AndreiChis 8/30/2013 14:39'!
defaultOrder

	^ 100! !

!GTChangeDebuggerAction methodsFor: 'actions' stamp: 'AndreiChis 7/14/2013 23:54'!
executeAction

	 self debugger debug: self interruptedContext using: debuggerClass! !

!GTChangeDebuggerAction methodsFor: 'initialization' stamp: 'AndreiChis 1/10/2016 15:39'!
initialize
	super initialize.
	self needsUpdate: false! !

DebugAction subclass: #GTClearEditorDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTClearEditorDebugAction commentStamp: 'TudorGirba 1/10/2016 23:38' prior: 0!
This action is responsible for canceling/clearing the changes in a code editor from a debugger.!

!GTClearEditorDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 1/17/2016 19:03'!
gtActionFor: aDebugger
	<gtContextCodeDebuggingAction>
	
	^ (self forDebugger: aDebugger)
		icon: GLMUIThemeExtraIcons glamorousCancel! !

!GTClearEditorDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 16:08'!
defaultIcon

	^ nil
	"^ Smalltalk ui icons smallCancelIcon"! !

!GTClearEditorDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 16:01'!
defaultKeyText

	^ 'l'! !

!GTClearEditorDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 15:59'!
defaultLabel

	^ 'Cancel'! !

!GTClearEditorDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/11/2016 15:18'!
defaultOrder

	^ 260! !

!GTClearEditorDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/17/2016 22:19'!
executeAction

	self debugger codePresentation hasUnacceptedEdits ifTrue: [  
		(self confirm:
'This action will cancel your changes.
Is it OK to cancel changes?' translated) ifTrue: [
		self debugger codePresentation clearUserEdits.
		self debugger updateCodeEditor ] ]! !

!GTClearEditorDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 15:59'!
id

	^ #gtClearEditorDebugAction! !

!GTClearEditorDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 1/10/2016 15:38'!
initialize
	super initialize.
	self needsUpdate: false! !

DebugAction subclass: #GTCodeDebugAction
	instanceVariableNames: 'codePresentationSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTCodeDebugAction commentStamp: 'AndreiChis 1/20/2015 19:47' prior: 0!
A GTCodeDebugAction encapsulates the logic for accessing the codePresentation from a debugger.
!

!GTCodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2015 19:43'!
codePresentation
	^ self debugger perform: self codePresentationSelector! !

!GTCodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2015 19:34'!
codePresentationSelector
	^ codePresentationSelector! !

!GTCodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/20/2015 19:35'!
codePresentationSelector: aSelector
	codePresentationSelector := aSelector! !

!GTCodeDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 1/20/2015 19:36'!
initialize
	super initialize.
	self codePresentationSelector: #codePresentation! !

GTCodeDebugAction subclass: #GTDebugSelectionDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTDebugSelectionDebugAction commentStamp: 'TudorGirba 1/10/2016 23:42' prior: 0!
This action encapsulates the logic of debugging a selected piece of code from the code pane of the debugger.!

!GTDebugSelectionDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 1/11/2016 15:59'!
gtActionFor: aDebugger
	"<gtContextCodeDebuggingAction>"
	
	^ (self forDebugger: aDebugger)! !

!GTDebugSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/8/2016 15:42'!
defaultKeyText

	^ 'D'! !

!GTDebugSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/6/2014 15:21'!
defaultLabel

	^ 'Debug it'! !

!GTDebugSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/6/2014 15:22'!
defaultOrder

	^ 20! !

!GTDebugSelectionDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/20/2015 19:43'!
executeAction
	
	self codePresentation debugSelection
		
! !

!GTDebugSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/6/2014 15:22'!
id

	^ #gTDebugSelectionDebugAction! !

!GTDebugSelectionDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 1/20/2015 19:36'!
initialize
	super initialize.
	self needsUpdate: false! !

GTCodeDebugAction subclass: #GTExecuteSelectionDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTExecuteSelectionDebugAction commentStamp: 'TudorGirba 1/10/2016 23:42' prior: 0!
This action is responsible for executing a selected piece of code from the code pane of the debugger.!

!GTExecuteSelectionDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 1/11/2016 15:59'!
gtActionFor: aDebugger
	"<gtContextCodeDebuggingAction>"
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousSpawn! !

!GTExecuteSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 4/17/2014 00:27'!
defaultKeyText

	^ 'd'! !

!GTExecuteSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 4/17/2014 00:28'!
defaultLabel

	^ 'Do it'! !

!GTExecuteSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/11/2016 15:38'!
defaultOrder

	^ 10! !

!GTExecuteSelectionDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/20/2015 19:44'!
executeAction
	
	self codePresentation evaluateSelection
		
! !

!GTExecuteSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 4/17/2014 00:27'!
id

	^ #gTExecuteSelectionDebugAction! !

!GTExecuteSelectionDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 1/20/2015 19:37'!
initialize
	super initialize.
	self needsUpdate: false! !

GTCodeDebugAction subclass: #GTInspectSelectionDebugAction
	instanceVariableNames: 'shouldBasicInspect'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTInspectSelectionDebugAction commentStamp: 'TudorGirba 1/10/2016 23:42' prior: 0!
This action is responsible for inspecting a selected piece of code from the code pane of the debugger.!

!GTInspectSelectionDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 1/11/2016 15:59'!
gtActionFor: aDebugger
	"<gtContextCodeDebuggingAction>"
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousInspect! !

!GTInspectSelectionDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 1/11/2016 15:59'!
gtBasicInspectActionFor: aDebugger
	"<gtContextCodeDebuggingAction>"
	
	^ (self forDebugger: aDebugger)
			makeBasicInspect;
			label: 'Basic inspect it';
			keyText: 'I'; 
			order: 16;
			icon: GLMUIThemeExtraIcons glamorousInspect! !

!GTInspectSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 4/17/2014 10:23'!
defaultKeyText

	^ 'i'! !

!GTInspectSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 4/17/2014 10:24'!
defaultLabel

	^ 'Inspect it'! !

!GTInspectSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/7/2014 11:09'!
defaultOrder

	^ 15! !

!GTInspectSelectionDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/11/2016 15:41'!
executeAction
	
	self codePresentation evaluateSelectionAndDo: [ :result |
		shouldBasicInspect 
			ifTrue: [ result basicInspect ]
			ifFalse: [ result inspect ] ]
		
! !

!GTInspectSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 4/17/2014 10:23'!
id

	^ #gTInspectSelectionDebugAction! !

!GTInspectSelectionDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 1/11/2016 15:41'!
initialize
	super initialize.
	self needsUpdate: false.
	shouldBasicInspect := false! !

!GTInspectSelectionDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/11/2016 15:40'!
makeBasicInspect

	shouldBasicInspect := true! !

GTCodeDebugAction subclass: #GTPrintSelectionDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTPrintSelectionDebugAction commentStamp: 'TudorGirba 1/10/2016 23:43' prior: 0!
This action is responsible for printing a selected piece of code from the code pane of the debugger.!

!GTPrintSelectionDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 1/11/2016 15:59'!
gtActionFor: aDebugger
	"<gtContextCodeDebuggingAction>"
	
	^ (self forDebugger: aDebugger)
		! !

!GTPrintSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/7/2014 11:05'!
defaultKeyText

	^ 'p'! !

!GTPrintSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/7/2014 11:05'!
defaultLabel

	^ 'Print it'! !

!GTPrintSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/11/2016 15:38'!
defaultOrder

	^ 15! !

!GTPrintSelectionDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/20/2015 19:44'!
executeAction
	
	self codePresentation printSelection
		
! !

!GTPrintSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/7/2014 11:05'!
id

	^ #gTPrintSelectionDebugAction! !

!GTPrintSelectionDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 1/20/2015 19:45'!
initialize
	super initialize.
	self needsUpdate: false! !

GTCodeDebugAction subclass: #GTProfileSelectionDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTProfileSelectionDebugAction commentStamp: 'TudorGirba 1/10/2016 23:43' prior: 0!
This action is responsible for profiling  a selected piece of code from the code pane of the debugger.!

!GTProfileSelectionDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 1/11/2016 15:59'!
gtActionFor: aDebugger
	"<gtContextCodeDebuggingAction>"
	
	^ (self forDebugger: aDebugger)! !

!GTProfileSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/6/2014 15:26'!
defaultLabel

	^ 'Profile it'! !

!GTProfileSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/7/2014 11:07'!
defaultOrder

	^ 30! !

!GTProfileSelectionDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/20/2015 19:44'!
executeAction
	
	self codePresentation profileSelection
		
! !

!GTProfileSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/6/2014 15:27'!
id

	^ #gTProfileSelectionDebugAction! !

!GTProfileSelectionDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 1/20/2015 19:45'!
initialize
	super initialize.
	self needsUpdate: false! !

GTCodeDebugAction subclass: #GTSaveDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTSaveDebugAction commentStamp: 'TudorGirba 1/10/2016 23:39' prior: 0!
This debugger action is responsible for saving the current method from a debugger.!

!GTSaveDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 1/17/2016 19:03'!
gtActionFor: aDebugger
	<gtContextCodeDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousAccept! !

!GTSaveDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/6/2014 17:46'!
defaultKeyText

	^ 's'! !

!GTSaveDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 8/30/2013 14:42'!
defaultLabel

	^ 'Accept'! !

!GTSaveDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 1/11/2016 15:19'!
defaultOrder

	^ 250! !

!GTSaveDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/17/2016 17:12'!
executeAction

	self needsUpdate: false.
	self codePresentation hasUnacceptedEdits 
		ifTrue: [
			self codePresentation 
				compileTextIn: self currentContext 
				from: self session 
				andDo: [ self needsUpdate: true ] ].
	self codePresentation flash! !

!GTSaveDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 11/6/2015 18:02'!
id

	^ #gTSaveDebugAction! !

GTCodeDebugAction subclass: #GTSelectionGoDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTSelectionGoDebugAction commentStamp: 'TudorGirba 1/10/2016 23:41' prior: 0!
This action is responsible for evaluating the selection in a code pane of a debugger and pushing the result in the inspector of the debugger.!

!GTSelectionGoDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 5/5/2014 18:36'!
gtActionFor: aDebugger
	<gtContextCodeDebuggingAction>
	
	^ (self forDebugger: aDebugger)! !

!GTSelectionGoDebugAction methodsFor: 'accessing' stamp: 'TudorGirba 10/6/2014 07:31'!
defaultKeyText

	^ 'g'! !

!GTSelectionGoDebugAction methodsFor: 'accessing' stamp: 'TudorGirba 10/6/2014 07:30'!
defaultLabel

	^ 'Do it and go'! !

!GTSelectionGoDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/6/2014 14:47'!
defaultOrder

	^ 5! !

!GTSelectionGoDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/20/2015 19:44'!
executeAction
	
	self codePresentation 
		evaluateSelectionAndDo: [ :result | self debugger inspect: result ] 
		
! !

!GTSelectionGoDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/5/2014 11:51'!
id

	^ #gTOpenSelectionDebugAction! !

!GTSelectionGoDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 1/20/2015 19:45'!
initialize
	super initialize.
	self needsUpdate: false! !

DebugAction subclass: #GTGlamourWrapperDebuggingAction
	instanceVariableNames: 'glamourAction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTGlamourWrapperDebuggingAction commentStamp: 'AndreiChis 1/11/2016 16:05' prior: 0!
I simply wrap an existing glamour action. Currently I just return the action when one send #asGlamourAction.!

!GTGlamourWrapperDebuggingAction class methodsFor: 'registration' stamp: 'AndreiChis 1/17/2016 19:12'!
gtActionFor: aDebugger
	<gtContextCodeDebuggingAction>
	|index|
	
	index := 0.
	^ aDebugger codePresentation defaultSelectionActions collect: [ :each | 
			self new 
				glamourAction: each;
				order: (index := index + 10) ]! !

!GTGlamourWrapperDebuggingAction methodsFor: 'transformation' stamp: 'AndreiChis 1/11/2016 15:57'!
asGlamourAction
	^ glamourAction! !

!GTGlamourWrapperDebuggingAction methodsFor: 'accessing' stamp: 'AndreiChis 1/11/2016 16:01'!
glamourAction: anObject
	glamourAction := anObject.! !

DebugAction subclass: #GTHelpDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTHelpDebugAction commentStamp: 'AndreiChis 1/25/2016 11:04' prior: 0!
I open a HelpBrowser for the currently active debugger.

To specify a help topic a debugger has to override the class side method #helpClass.!

!GTHelpDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 1/25/2016 10:15'!
gtDebuggingAction
	"<gtDebuggingAction>"! !

!GTHelpDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/24/2016 23:52'!
defaultIcon

	^ GLMUIThemeExtraIcons glamorousHelp! !

!GTHelpDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/24/2016 23:55'!
defaultLabel

	^ ''! !

!GTHelpDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 1/25/2016 10:13'!
defaultOrder

	^ 200! !

!GTHelpDebugAction methodsFor: 'actions' stamp: 'AndreiChis 1/25/2016 10:53'!
executeAction

	HelpBrowser open selectTopicSatisfying: [:each | 
		each owner notNil and:  [
			(each owner key = self debugger class helpClass key) and: [
				each title = 'Overview' ] ] ]! !

!GTHelpDebugAction methodsFor: 'initialization' stamp: 'AndreiChis 1/24/2016 23:51'!
initialize
	super initialize.
	self needsUpdate: false! !

DebugAction subclass: #GTTemporaryPreDebugChangeDebuggerAction
	instanceVariableNames: 'preDebugWindow'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTTemporaryPreDebugChangeDebuggerAction commentStamp: 'TudorGirba 1/10/2016 23:45' prior: 0!
This action adds the possibility of opening a GTDebugger from the PreDebugWindow.!

!GTTemporaryPreDebugChangeDebuggerAction class methodsFor: 'registration' stamp: 'AndreiChis 11/13/2013 23:09'!
actionType
	<preDebuggingAction>! !

!GTTemporaryPreDebugChangeDebuggerAction methodsFor: 'accessing' stamp: 'AndreiChis 11/13/2013 22:55'!
defaultLabel

	^ 'Debug Glamorously'! !

!GTTemporaryPreDebugChangeDebuggerAction methodsFor: 'accessing' stamp: 'AndreiChis 11/13/2013 22:55'!
defaultOrder

	^ 200! !

!GTTemporaryPreDebugChangeDebuggerAction methodsFor: 'actions' stamp: 'AndreiChis 12/17/2013 11:24'!
executeAction
	| currentDebugger currentSession |
	
	"Close the spec PreDebugWindow and debugger"
	currentDebugger := self preDebugWindow debugger.
	self preDebugWindow debugger: nil.
	self preDebugWindow close.
	currentSession := currentDebugger session.
	currentDebugger session: nil.
	currentDebugger clear. "clear not close, as at this point there is no open window to close"
	
	GTGenericStackDebugger debugSession: currentSession! !

!GTTemporaryPreDebugChangeDebuggerAction methodsFor: 'accessing' stamp: 'AndreiChis 11/13/2013 22:55'!
id

	^ #gtChangeDebuggerPreDebug! !

!GTTemporaryPreDebugChangeDebuggerAction methodsFor: 'accessing' stamp: 'AndreiChis 11/13/2013 22:59'!
preDebugWindow
	
	^ preDebugWindow! !

!GTTemporaryPreDebugChangeDebuggerAction methodsFor: 'accessing' stamp: 'AndreiChis 11/13/2013 22:59'!
preDebugWindow: anObject
	
	preDebugWindow := anObject! !

Object subclass: #GTDebuggerContextPredicate
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Model'!

!GTDebuggerContextPredicate commentStamp: 'AndreiChis 12/30/2015 10:17' prior: 0!
I encapsulate a condition that can be verified against a Context.

To add concrete conditions a subclass should be create that overrides the method #matchContext:

Public API and Key Messages
- #matches: 
- #hasMatched
- #result

Instance Variables
result:		the last value returned by matches:

!

!GTDebuggerContextPredicate methodsFor: 'testing' stamp: 'AndreiChis 7/17/2013 19:48'!
hasMatched

	^ self result ifNil: [ ^ false ]! !

!GTDebuggerContextPredicate methodsFor: 'testing' stamp: 'AndreiChis 8/6/2013 17:36'!
matchContext: aContext
	"Subclasses should implement this method to add concrete conditions. "
	
	self subclassResponsibility! !

!GTDebuggerContextPredicate methodsFor: 'testing' stamp: 'AndreiChis 8/6/2013 17:34'!
matches: aContext
	"Return true if the given context satisfies the condition contained by this predicate,
	or false otherwise."
	
	^ result := self matchContext: aContext! !

!GTDebuggerContextPredicate methodsFor: 'accessing' stamp: 'AndreiChis 8/6/2013 17:35'!
result
	"The value returned by the most recent call to matches: "
	
	^ result! !

Object subclass: #GTDebuggerSystemSettings
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Model'!

!GTDebuggerSystemSettings commentStamp: 'AndreiChis 1/7/2016 10:10' prior: 0!
I register settings only needed for GTDebugger.!

!GTDebuggerSystemSettings class methodsFor: 'settings' stamp: 'AndreiChis 1/7/2016 10:08'!
debugSettingsOn: aBuilder
	<systemsettings>
	
	(aBuilder setting: #enableStackColoring)
		label: 'Color the stack' translated;
		target: GTGenericStackDebugger;
		parent: #debugging;
		description:
				'When true, the stack context are colored based on their relationship with the selected context.' translated.
	
	(aBuilder setting: #enableDebuggerWindowDistinctColor)
		label: 'Color the window' translated;
		target: GTGenericStackDebugger;
		parent: #debugging;
		description:
				'When true, the window border is colored with red to make it distinct from other windows.' translated.
				
	(aBuilder setting: #GTDebuggerStatus)
		parent: #gt;
		target: GTGenericStackDebugger;
		getSelector: #isGTDebuggerEnabled;
		setSelector: #setGTDebuggerEnabledStatus:; 
		label: 'GTDebugger';
		description: 
			'Enable or disable the GTDebugger.'
	
! !

!Object methodsFor: '*GT-Debugger-UI' stamp: 'AndreiChis 1/10/2016 18:46'!
gtConstructDebuggerInspectorIn: composite for: aDebugger
	
	^ composite custom: (GTDebuggerInspector new 
		debugger: aDebugger;
		startOn: self)! !

!Object methodsFor: '*GT-Debugger-UI' stamp: 'AndreiChis 1/15/2016 17:36'!
gtDebuggerEvaluatorIn: composite
	
	(GTObjectVariablesBrowser new evaluatorIn: composite)
		title: 'Evaluator'! !

!Object methodsFor: '*GT-Debugger-UI' stamp: 'TudorGirba 1/17/2016 22:40'!
gtDebuggerRawIn: composite
	
	(GTObjectVariablesBrowser new variablesIn: composite)
		title: 'Raw';
		send: #value! !

!SpecDebugger class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 12/29/2015 17:39'!
availableAutomatically

	^ false! !

!SpecDebugger class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 12/29/2015 16:39'!
handlesContext: aContext

	^ true! !

!SpecDebugger class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 12/29/2015 16:40'!
spanNewSessionFrom: anotherSession

	^ self sessionClass 
		named: anotherSession name 
		on: anotherSession interruptedProcess 
		startedAt: anotherSession interruptedContext! !

!ResumeDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/10/2016 15:49'!
gtGlobalDebuggingActionFor: aDebugger
	<gtDebuggingAction>
	
	^ (self forDebugger: aDebugger)
		label: nil! !

!ResumeDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/10/2016 15:49'!
gtStackDebuggingActionFor: aDebugger
	<gtStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousPlay! !

!CopyToClipboardDebugAction class methodsFor: '*GT-Debugger-Actions' stamp: 'AndreiChis 9/12/2014 15:16'!
gtActionFor: aDebugger
	<gtStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			order: 35! !

!DebugContext methodsFor: '*gt-debugger' stamp: 'VincentBlondeau 2/14/2014 16:47'!
gtDebuggerRetrieveVariableValuePairs
	| fieldList |
	fieldList := OrderedCollection new.
	fieldList
		add: '_self' -> context receiver;
		addAll: (context receiver class allInstVarNames collect: [ :each | each -> (context receiver instVarNamed: each) ]).
	fieldList add: '_thisContext' -> context.
	context == nil
		ifTrue: [ ^ fieldList ].
	fieldList
		add:
			'_stack top'
				->
					(context stackPtr > 0
						ifTrue: [ context top ]).
	context tempNames notEmpty
		ifTrue: [ 
			fieldList
				addAll:
					((1 to: context tempNames size)
						collect: [ :index | 
							| key value |
							key := context tempNames at: index.
							value := [ context debuggerMap namedTempAt: index in: context ]
								on: Error
								do: [ value := (Text string: 'error obtaining field value' attribute: TextColor red) asString ].
							key -> value ]) ].
	^ fieldList! !

!ReturnValueDebugAction class methodsFor: '*GT-Debugger-Actions' stamp: 'AndreiChis 1/20/2015 19:08'!
gtActionType
	<gtContextStackDebuggingAction>! !

!StepIntoDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/10/2016 15:46'!
gtGlobalDebuggingActionFor: aDebugger
	<gtDebuggingAction>
	
	^ (self forDebugger: aDebugger)
		label: nil! !

!StepIntoDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/10/2016 15:47'!
gtStackDebuggingActionFor: aDebugger
	<gtStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousInto! !

!DoesNotUnderstandDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 2/17/2016 14:52'!
gtStackDebuggingActionFor: aDebugger
	<gtStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousAdd;
			order: 4! !

!SubclassResponsabilityDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/10/2016 23:59'!
gtStackDebuggingActionFor: aDebugger
	<gtStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousAdd! !

!PeelToFirstDebugAction class methodsFor: '*GT-Debugger-Actions' stamp: 'AndreiChis 1/20/2015 19:08'!
gtActionType
	<gtContextStackDebuggingAction>! !

BrowseDebugAction subclass: #GTBrowseDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTBrowseDebugAction commentStamp: 'AndreiChis 12/30/2015 11:13' prior: 0!
I am debugging action for sending a unary message to the debugger.!

!GTBrowseDebugAction class methodsFor: 'actions creation' stamp: 'AndreiChis 1/20/2015 19:07'!
gtDebugActionsFor: aDebugger
	<gtContextStackDebuggingAction>
	
	^ super debugActionsFor: aDebugger! !

!GTBrowseDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 4/16/2014 17:54'!
receiver

	^ self debugger! !

!StepThroughDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/10/2016 15:48'!
gtGlobalDebuggingActionFor: aDebugger
	<gtDebuggingAction>
	
	^ (self forDebugger: aDebugger)
		label: nil! !

!StepThroughDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/10/2016 15:48'!
gtStackDebuggingActionFor: aDebugger
	<gtStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousThrough! !

!Context methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/5/2015 16:53'!
findContextMatchingPredicate: aPredicate
	^ self findContextSuchThat: [ :ctx |
		aPredicate matches: ctx ]! !

!Context methodsFor: '*GT-Debugger' stamp: 'TudorGirba 1/17/2016 22:33'!
gtConstructDebuggerInspectorIn: composite for: aDebugger
	
	^composite custom: ((GTDebuggerInspector new  
		debugger: aDebugger;
		notifyPaneRemoved: true;
		showFirst: [ :a | a custom: (aDebugger class variablesBrowserClass new 
			debugger: aDebugger;
			title: 'Variables';
			startOn: self)];
		yourself) 
			startOn: self)! !

!Context methodsFor: '*GT-Debugger' stamp: 'TudorGirba 1/9/2016 21:05'!
gtDebuggerRetrieveVariableValuePairs
	|fieldList context|

	fieldList := SortedCollection 
						sortBlock: [:variablePair1 :variablePair2 | 
							variablePair1 key < variablePair2 key].
	context := self.
	fieldList 
		addAll: (context receiver class allSlots collect: [:slot ||value| 
				value := [(slot read: context receiver)] 
					on: Error 
					do: [ value := (Text string: 'error obtaining field value' attribute: TextColor red) asString ].
			slot name -> value ] ).
	
	context == nil ifTrue: [^fieldList].
		
	(context tempNames notEmpty) ifTrue: [
		fieldList addAll: ((1 to: context tempNames size) collect: [ :index || key value |
			key := context tempNames at: index.
			value := [context debuggerMap namedTempAt: index in: context] 
				on: Error 
				do: [value := (Text string: 'error obtaining field value' attribute: TextColor red) asString ].
			key -> value ] ) ].
	
	^ fieldList asOrderedCollection 
		addFirst: ('self' ->  context receiver);
		yourself
! !

!Context methodsFor: '*GT-Debugger' stamp: 'TudorGirba 1/12/2016 01:09'!
outerMostContext
	^ self outerContext 
		ifNil: [ self ]
		ifNotNil: [ self outerContext ]! !

!Context methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/9/2016 20:48'!
reservedNames

	^ #( 'self')! !

RunToSelectionDebugAction subclass: #GTRunToSelectionDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTRunToSelectionDebugAction commentStamp: 'TudorGirba 1/10/2016 23:41' prior: 0!
This action is responsible for continuing the execution up to the point selected in the code pane.!

!GTRunToSelectionDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 1/11/2016 15:03'!
gtActionFor: aDebugger
	<gtContextCodeDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousPlay! !

!GTRunToSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 5/6/2014 15:24'!
defaultOrder

	^ 1! !

!GTRunToSelectionDebugAction methodsFor: 'actions' stamp: 'AndreiChis 12/16/2013 15:32'!
executeAction

	self session 
		runToSelection: self debugger selectionInterval
		inContext: self currentContext! !

!GTRunToSelectionDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 15:29'!
id

	^ #gtRunToSelection! !

!RunToSelectionDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 11/13/2013 23:46'!
gtActionFor: aDebugger
	"<gtCodeDebuggingAction>"
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousPlay! !

!ProtoObject methodsFor: '*GT-Debugger-UI' stamp: 'AndreiChis 1/15/2016 17:37'!
gtDebuggerEvaluatorIn: composite
	<gtDebuggerPresentationOrder: 10>
	
	(GTProtoObjectVariablesBrowser new evaluatorIn: composite)
		title: 'Evaluator'! !

!ProtoObject methodsFor: '*GT-Debugger-UI' stamp: 'AndreiChis 1/22/2016 16:32'!
gtDebuggerPresentationsIn: composite inContext: aGTInspector
	| pragmas |
	pragmas := (Pragma 
				allNamed: #gtDebuggerPresentationOrder:
				from: self class 
				to: ProtoObject) asOrderedCollection.
	pragmas addAll:((Pragma 
				allNamed: aGTInspector class extensionsPragma 
				from: self class 
				to: ProtoObject)
					reject: [:pragma| pragma method selector = #gtInspectorRawIn: ]).
	pragmas := pragmas sorted: [: x :y | (x argumentAt: 1) < (y argumentAt: 1)].
	self gtInspectorPresentationsFromPragmas: pragmas In: composite inContext: aGTInspector
! !

!ProtoObject methodsFor: '*GT-Debugger-UI' stamp: 'TudorGirba 1/17/2016 22:40'!
gtDebuggerRawIn: composite
	<gtDebuggerPresentationOrder: 15>
	
	(GTProtoObjectVariablesBrowser new variablesIn: composite)
		title: 'Raw';
		send: #value! !

!Pharo3DarkTheme methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/8/2016 19:43'!
sameClassContextStyleFor: aContext 

	^ {TextColor cyan}! !

!Pharo3DarkTheme methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/8/2016 19:43'!
samePackageContextStyleFor: aContext

	^ {TextColor magenta}! !

!Pharo3DarkTheme methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/8/2016 19:44'!
unrelatedContextStyleFor: aContext

	^ {TextColor color: Color white}! !

!StepOverDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/10/2016 15:47'!
gtGlobalDebuggingActionFor: aDebugger
	<gtDebuggingAction>
	
	^ (self forDebugger: aDebugger)
		label: nil! !

!StepOverDebugAction class methodsFor: '*GT-Debugger' stamp: 'AndreiChis 1/10/2016 15:48'!
gtStackDebuggingActionFor: aDebugger
	<gtStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousOver! !

!DebugSession methodsFor: '*gt-debugger' stamp: 'AndreiChis 8/14/2013 18:21'!
isActive: aContext

	^ true! !

WhereIsDebugAction subclass: #GTWhereIsDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Actions'!

!GTWhereIsDebugAction commentStamp: 'TudorGirba 1/10/2016 23:39' prior: 0!
This debugger action is responsible for reselecting the code corresponding with the current instruction to be executed.!

!GTWhereIsDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 11/13/2013 23:40'!
gtActionFor: aDebugger
	<gtCodeDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousSearch! !

!GTWhereIsDebugAction methodsFor: 'accessing' stamp: 'TudorGirba 1/10/2016 19:09'!
defaultOrder

	^ 200! !

!GTWhereIsDebugAction methodsFor: 'actions' stamp: 'AndreiChis 11/13/2013 23:39'!
executeAction

	self debugger updateSelectionInterval! !

!GTWhereIsDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 11/13/2013 23:39'!
id

	^ #gtWhereIs! !

!FuelOutStackDebugAction class methodsFor: '*GT-Debugger-Actions' stamp: 'AndreiChis 9/12/2014 15:00'!
gtActionFor: aDebugger
	<gtStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			order: 30! !

!ToolRegistry methodsFor: '*GT-Debugger-Model' stamp: 'AndreiChis 12/28/2015 20:31'!
debuggerDispatcherFor: aContext matching: aBlock
	| debuggers |
	debuggers := self debuggersWithRanksMatching: [ :aDebugger | 
		(aBlock value: aDebugger) and: [ 
			aDebugger handlesContext: aContext ] ].
	(debuggers size > 0) ifTrue: [ ^ (debuggers at: 1) key ].
	^ nil! !

!ToolRegistry methodsFor: '*GT-Debugger-Model' stamp: 'AndreiChis 11/27/2015 18:44'!
debuggersForContext: aContext

	^ self debuggersMatching: [ :aDebugger | aDebugger handlesContext: aContext ]! !

!ToolRegistry methodsFor: '*GT-Debugger-Model' stamp: 'AndreiChis 11/27/2015 18:35'!
debuggersMatching: aBlock

	^ (self debuggersWithRanksMatching: aBlock) 
			inject: OrderedCollection new
			into: [ :aCollection :aPair | aCollection add: aPair key; yourself ]! !

!ToolRegistry methodsFor: '*GT-Debugger-Model' stamp: 'AndreiChis 12/28/2015 20:31'!
debuggersWithRanksMatching: aBlock

	^ ((self registeredDebuggersWithRanks) 
			select: [ :aPair | aBlock value: aPair key ])
			sort: [ :pair1 :pair2 | pair1 value > pair2 value ].! !

!ToolRegistry methodsFor: '*GT-Debugger-Model' stamp: 'AndreiChis 4/2/2013 21:49'!
registerDebugger: anObject withRank: anInteger
	|debuggers|
	
	debuggers  := tools at: #debuggers ifAbsent: [
		tools at: #debuggers put: OrderedCollection new ]. 
	(debuggers anySatisfy: [:each| each key == anObject] ) ifFalse: [ 
		debuggers add: anObject -> anInteger ].
	WorldState defaultWorldMenu! !

!ToolRegistry methodsFor: '*GT-Debugger-Model' stamp: 'AndreiChis 4/2/2013 21:48'!
registeredDebuggersWithRanks
	
	^ tools at: #debuggers ifAbsent: [
		tools at: #debuggers put: OrderedCollection new ]! !

CustomHelp subclass: #GTDebuggerHelp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-Model'!

!GTDebuggerHelp commentStamp: 'AndreiChis 1/25/2016 11:07' prior: 0!
A utility class containing the help for GT Debugger.

Customs debuggers that want to provide a help topic should create a subclass and override GTMoldableDebugger class>>#helpClass.

I dynamically build and display  the list of actions  available for a debugger.!

!GTDebuggerHelp class methodsFor: 'documentation' stamp: 'AndreiChis 1/25/2016 11:10'!
actions
		
	^ HelpTopic 
		title: 'Actions'
		contents: 'TODO'! !

!GTDebuggerHelp class methodsFor: 'accessing' stamp: 'AndreiChis 1/24/2016 23:44'!
bookName 
	^'GT Debugger'! !

!GTDebuggerHelp class methodsFor: 'accessing' stamp: 'AndreiChis 1/24/2016 23:44'!
key
	^ self name! !

!GTDebuggerHelp class methodsFor: 'documentation' stamp: 'AndreiChis 1/25/2016 11:10'!
overview
	^ HelpTopic 
		title: 'Overview'
		contents: 
'GT Debugger is a moldable debugger that is part of Pharo and was developed as part of the Glamorous Toolkit project (http://gt.moosetechnology.org)'! !

!GTDebuggerHelp class methodsFor: 'accessing' stamp: 'AndreiChis 1/25/2016 11:10'!
pages
	^#(overview actions)! !

GTInspector subclass: #GTDebuggerInspector
	instanceVariableNames: 'debugger notifyPaneRemoved'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Debugger-UI'!

!GTDebuggerInspector commentStamp: 'AndreiChis 12/30/2015 10:03' prior: 0!
This is a specialized inspector that is embedded in  GTDebugger.

It is special in two ways:
- the first presentation is different in that it shows all variables active in the debugging context
- the looking up of presentation ignores the default state presentation because the vertical splitting of state and workspace does not work in the context of the debugger due to space limitations. Instead, the workspace is offered in a separate presentation.!

!GTDebuggerInspector methodsFor: 'callbacks' stamp: 'AndreiChis 1/10/2016 14:19'!
actOnPaneAdded: ann

	self panes size > 1 ifTrue: [ 
		(self panes before: self panes last) presentations first updateToolbar ]! !

!GTDebuggerInspector methodsFor: 'callbacks' stamp: 'AndreiChis 1/10/2016 18:59'!
actOnPaneRemoved: ann

	self first panes last presentations first updateToolbar.
	self notifyPaneRemoved ifTrue: [ 
		self debugger announce: ann ]! !

!GTDebuggerInspector methodsFor: 'building' stamp: 'AndreiChis 1/10/2016 14:43'!
attachExtraBehaviourToPresentationsIn: aComposite

	aComposite presentations do: [ :aPresentation |
		aPresentation addAction: 
			((self closeActionFor: aPresentation)
				condition: [ :currentPresentation |
					self panes size > 1 and: [ 
						self isPresentation: aPresentation inPane: self panes last  ] ] ) ]! !

!GTDebuggerInspector methodsFor: 'building' stamp: 'AndreiChis 1/10/2016 18:51'!
compose
	self pager
		minimumPageNumberForScrollbar: 2; 
		show: [ :a :each | 
			a dynamic display: [ :x | 
				GLMCompositePresentation new with: [ :aComposite | 
					each gtDebuggerPresentationsIn: aComposite inContext: self.
					self attachExtraBehaviourToPresentationsIn: aComposite.
					aComposite ] ] ].
	self first when: GLMPaneAdded send: #actOnPaneAdded: to: self.
	self first when: GLMPaneRemoved send: #actOnPaneRemoved: to: self! !

!GTDebuggerInspector methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 18:46'!
debugger
	^ debugger! !

!GTDebuggerInspector methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 18:46'!
debugger: anObject
	debugger := anObject! !

!GTDebuggerInspector methodsFor: 'building' stamp: 'AndreiChis 1/10/2016 14:40'!
isPresentation: aPresentation inPane: aPane
	^ aPane presentations first cachedPresentation 
		ifNil: [ false ] 
		ifNotNil: [ :aCachedPresentation | aCachedPresentation first includes: aPresentation ]! !

!GTDebuggerInspector methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 18:58'!
notifyPaneRemoved
	^ notifyPaneRemoved ifNil: [ false ]! !

!GTDebuggerInspector methodsFor: 'accessing' stamp: 'AndreiChis 1/10/2016 18:56'!
notifyPaneRemoved: anObject
	notifyPaneRemoved := anObject! !
